package net.jaekl.cfb.db;

// Copyright (C) 2015 Christian Jaekl

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Locale;

import net.jaekl.cfb.db.driver.DbDriver;

public class Schema {
	String m_name;
	DbDriver m_driver;
	ArrayList<Table> m_tables;
	
	public Schema(String name, DbDriver driver) {
		m_name = name;
		m_driver = driver;
		m_tables = new ArrayList<Table>();
	}
	
	public boolean ensureDbInitialized(Connection con) throws SQLException {
		assert(null != con);
		
		if (allTablesPresent(con)) {
			return true;
		}
		
		if (!createAllTables(con)) {
			return false;
		}
		
		return true;
	}
	
	boolean allTablesPresent(Connection con) throws SQLException 
	{
		assert(null != con);
		
		DatabaseMetaData dbmd = con.getMetaData();
		HashSet<String> extantTables = new HashSet<String>();
		
		try (ResultSet rs = dbmd.getTables(null, null, null, new String[]{"TABLE"})) {
			while (rs.next()) {
				extantTables.add(rs.getString(3).toUpperCase(Locale.CANADA));
			}
		}
		
		for (Table table : m_tables) {
			String name = table.getName().toUpperCase(Locale.CANADA);
			if ( ! extantTables.contains(name) ) {
				// One or more tables missing
				return false;
			}
		}
		
		// We could be more thorough here, and check that the expected columns are in place.
		// Also, eventually, some sort of DB schema versioning will be needed.
		
		return true;		
	}
	
	boolean createAllTables(Connection con) throws SQLException {
		for (Table table : m_tables) {
			if (!m_driver.createTable(con, table)) {
				return false;
			}
		}
		return true;
	}
	
	void addTable(Table table) {
		m_tables.add(table);
	}
	
	// Add a list of tables.
	// Define each table in the list as follows:
	// {
	//   { table_name },
	//   { column_name, type, width (-1 for default), null/not_null }
	// }
	void addTables(Object[][][] tables) 
	{
		for (Object[][] table : tables) {
			addTable(Table.construct(table));
		}
	}	
}
