package net.jaekl.cfb.db.driver;

// Copyright (C) 2015 Christian Jaekl

import static net.jaekl.cfb.db.Column.Null.*;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import net.jaekl.cfb.db.Column;
import net.jaekl.cfb.db.Column.Type;
import net.jaekl.cfb.db.Table;

public abstract class DbDriver {
	DbDriver() {
		
	}
	
	// Load the JDBC driver
	public abstract void load() throws ClassNotFoundException;
	
	public abstract Connection connect(String host, int port, String dbName, String user, String pass) throws SQLException;
	
	public boolean createTable(Connection con, Table table) throws SQLException {
		String sql = createTableSql(table);
		try (PreparedStatement ps = con.prepareStatement(sql)) {
			ps.executeUpdate();
		}
		catch (SQLException exc) {
			throw new SQLException("Failed to executeUpdate:  " + sql, exc);
		}
		
		return true;
	}
	
	public abstract ResultSet selectColumnsFromWhere(Column[] columns, Table[] tables, String where);
	
	protected String typeName(Type type) {
		return type.toString();
	}
	
	protected String createColumnSql(Column column) 
	{
		String result = column.getName() + " " + typeName(column.getType());
		if (column.getWidth() > 0) {
			result += "(" + column.getWidth() + ")";
		}
		
		if (NOT_NULL == column.getNull()) {
			result += " NOT NULL";
		}
		else {
			result += " NULL";
		}
	    
		return result;
	}
	
	protected String createTableSql(Table table) 
	{
		assert(null != table);
		assert(null != table.getName());
		assert(table.getNumColumns() > 0);
		
		StringBuilder sb = new StringBuilder();
		
		sb.append("CREATE TABLE ")
		  .append(table.getName())
		  .append("(");
		
		for (int idx = 0; idx < table.getNumColumns(); ++idx) {
			if (idx > 0) {
				sb.append(", ");
			}
			sb.append(createColumnSql(table.getColumn(idx)));
		}
		
		sb.append(")");
		
		return sb.toString();
	}	
}
